@@ -13,8 +13,8 @@ def test_constructor_empty():
     assert result.cell is None
 
 
-@pytest.mark.parametrize("spacegroup", [None, gemmi.SpaceGroup(1)])
-@pytest.mark.parametrize("cell", [None, gemmi.UnitCell(1, 1, 1, 90, 90, 90)])
+@pytest.mark.parametrize("spacegroup", [None, gemmi.SpaceGroup(1), "P 1"])
+@pytest.mark.parametrize("cell", [None, gemmi.UnitCell(1, 1, 1, 90, 90, 90), (1, 1, 1,  90, 90, 90)])
 @pytest.mark.parametrize("merged", [None, True, False])
 def test_constructor_dataset(data_fmodel, spacegroup, cell, merged):
     """Test DataSet.__init__() when called with a DataSet"""
@@ -28,12 +28,17 @@ def test_constructor_dataset(data_fmodel, spacegroup, cell, merged):
         
     # Ensure provided values take precedence
     if spacegroup:
-        assert result.spacegroup == spacegroup
+        assert result.spacegroup.xhm() == "P 1"
+        assert isinstance(result.spacegroup, gemmi.SpaceGroup)
     else:
         assert result.spacegroup == data_fmodel.spacegroup
 
     if cell:
-        assert result.cell == cell
+        if isinstance(cell, gemmi.UnitCell):
+            assert result.cell == cell
+        else:
+            assert result.cell.a == cell[0]
+        assert isinstance(result.cell, gemmi.UnitCell)
     else:
         assert result.cell == data_fmodel.cell
 
@@ -198,6 +203,7 @@ def test_label_centrics(data_fmodel, inplace, no_sg):
         assert "CENTRIC" in result
         assert result["CENTRIC"].dtype.name == "bool"
 
+
 @pytest.mark.parametrize("inplace", [True, False])
 @pytest.mark.parametrize("no_sg", [True, False])
 def test_label_absences(data_fmodel, inplace, no_sg):
@@ -220,6 +226,40 @@ def test_label_absences(data_fmodel, inplace, no_sg):
         assert result["ABSENT"].dtype.name == "bool"
 
 
+@pytest.mark.parametrize("cache", [True, False])
+def test_centrics(mtz_by_spacegroup, cache):
+    """Test DataSet.centrics against DataSet.label_centrics"""
+    ds =  rs.read_mtz(mtz_by_spacegroup)
+    if cache:
+        ds.label_centrics(inplace=True)
+        expected = ds.loc[ds.CENTRIC]
+        result = ds.centrics
+        assert "CENTRIC" in result
+    else:
+        expected = ds.loc[ds.label_centrics()["CENTRIC"]]
+        result = ds.centrics
+        assert "CENTRIC" not in result
+
+    assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize("cache", [True, False])
+def test_acentrics(mtz_by_spacegroup, cache):
+    """Test DataSet.acentrics against DataSet.label_centrics"""
+    ds =  rs.read_mtz(mtz_by_spacegroup)
+    if cache:
+        ds.label_centrics(inplace=True)
+        expected = ds.loc[~ds.CENTRIC]
+        result = ds.acentrics
+        assert "CENTRIC" in result
+    else:
+        expected = ds.loc[~ds.label_centrics()["CENTRIC"]]
+        result = ds.acentrics
+        assert "CENTRIC" not in result
+
+    assert_frame_equal(result, expected)
+
+
 @pytest.mark.parametrize("inplace", [True, False])
 @pytest.mark.parametrize("index", [True, False])
 def test_infer_mtz_dtypes(data_merged, inplace, index):
